==> sdk-4.1.0-sources.jar: com/kontakt/sdk/android/cloud/KontaktCloudImpl.java <==
package com.kontakt.sdk.android.cloud;

import android.content.Context;
import android.os.Build;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.kontakt.sdk.android.BuildConfig;
import com.kontakt.sdk.android.cloud.adapter.ActionTypeAdapter;
import com.kontakt.sdk.android.cloud.adapter.ConfigTypeAdapter;
import com.kontakt.sdk.android.cloud.adapter.DeviceTypeAdapter;
import com.kontakt.sdk.android.cloud.api.*;
import com.kontakt.sdk.android.cloud.api.service.*;
import com.kontakt.sdk.android.cloud.serialize.CoordinatesDeserializer;
import com.kontakt.sdk.android.cloud.serialize.CoordinatesSerializer;
import com.kontakt.sdk.android.cloud.util.ErrorUtils;
import com.kontakt.sdk.android.common.model.Action;
import com.kontakt.sdk.android.common.model.Config;
import com.kontakt.sdk.android.common.model.Coordinates;
import com.kontakt.sdk.android.common.model.Device;
import com.kontakt.sdk.android.common.util.SDKPreconditions;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import retrofit2.converter.scalars.ScalarsConverterFactory;

import java.io.IOException;
import java.util.Locale;

import static com.kontakt.sdk.android.cloud.util.StringUtils.isAsciiPrintable;

/**
 * Kontakt.io Proximity REST API client implementation.
 *
 * To obtain Kontakt Cloud instance please use {@link KontaktCloudFactory#create(String)} or
 * {@link KontaktCloudFactory#create()} method, for example:
 *
 * <pre>
 *   <code>
 *   KontaktCloud kontaktCloud = KontaktCloudFactory.create();
 *   KontaktCloud kontaktCloud = KontaktCloudFactory.create("my-api-key");
 *   </code>
 * </pre>
 */
@SuppressWarnings("WeakerAccess")
public class KontaktCloudImpl implements KontaktCloud {

  private final ServicesFactory servicesFactory;
  // used to get app package header value
  private Context context;

  KontaktCloudImpl(String apiKey, String apiUrl, int apiVersion) {
    SDKPreconditions.checkNotNullOrEmpty(apiKey, "Kontakt Cloud - empty api key.");
    SDKPreconditions.checkNotNullOrEmpty(apiUrl, "Kontakt Cloud - empty api url.");
    final Retrofit retrofit = buildRetrofit(apiUrl, buildHttpClient(apiKey, apiVersion));
    this.servicesFactory = ServicesFactory.newInstance(retrofit);
    ErrorUtils.initialize(retrofit);
  }

  private KontaktCloudImpl(Context context, String apiKey, String apiUrl, int apiVersion) {
    SDKPreconditions.checkNotNull(context, "Kontakt Cloud - null context.");
    SDKPreconditions.checkNotNullOrEmpty(apiKey, "Kontakt Cloud - empty api key.");
    SDKPreconditions.checkNotNullOrEmpty(apiUrl, "Kontakt Cloud - empty api url.");

    this.context = context;
    final Retrofit retrofit = buildRetrofit(apiUrl, buildHttpClient(apiKey, apiVersion));
    this.servicesFactory = ServicesFactory.newInstance(retrofit);
    ErrorUtils.initialize(retrofit);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ActionsApi actions() {
    return new ActionsApi(servicesFactory.createService(ActionsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DevicesApi devices() {
    return new DevicesApi(servicesFactory.createService(DevicesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public VenuesApi venues() {
    return new VenuesApi(servicesFactory.createService(VenuesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public PlacesApi places() {
    return new PlacesApi(servicesFactory.createService(PlacesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ReceiversApi receivers() {
    return new ReceiversApi(servicesFactory.createService(ReceiversService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ConfigsApi configs() {
    return new ConfigsApi(servicesFactory.createService(ConfigsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ManagersApi managers() {
    return new ManagersApi(servicesFactory.createService(ManagersService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public FirmwaresApi firmwares() {
    return new FirmwaresApi(servicesFactory.createService(FirmwaresService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ProximitiesApi proximities() {
    return new ProximitiesApi(servicesFactory.createService(ProximitiesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public NamespacesApi namespaces() {
    return new NamespacesApi(servicesFactory.createService(NamespacesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public CommandsApi commands() {
    return new CommandsApi(servicesFactory.createService(CommandsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public EventsApi events() {
    return new EventsApi(servicesFactory.createService(EventsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public PresetsApi presets() {
    return new PresetsApi(servicesFactory.createService(PresetsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public TriggersApi triggers() {
    return new TriggersApi(servicesFactory.createService(TriggersService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ActivitiesApi activities() {
    return new ActivitiesApi(servicesFactory.createService(ActivitiesService.class));
  }

  private NotificationsApi notificationsApi() {
    return new NotificationsApi(servicesFactory.createService(NotificationsService.class));
  }

  private Retrofit buildRetrofit(final String apiBaseUrl, final OkHttpClient okHttpClient) {
    GsonBuilder gsonBuilder = new GsonBuilder();
    gsonBuilder.registerTypeAdapter(Device.class, new DeviceTypeAdapter());
    gsonBuilder.registerTypeAdapter(Config.class, new ConfigTypeAdapter());
    gsonBuilder.registerTypeAdapter(Action.class, new ActionTypeAdapter());
    gsonBuilder.registerTypeAdapter(Coordinates.class, new CoordinatesSerializer());
    gsonBuilder.registerTypeAdapter(Coordinates.class, new CoordinatesDeserializer());
    gsonBuilder.setPrettyPrinting();
    Gson gson = gsonBuilder.create();

    Retrofit.Builder retrofitBuilder = new Retrofit.Builder()
            .baseUrl(apiBaseUrl)
            .addConverterFactory(ScalarsConverterFactory.create())
            .addConverterFactory(GsonConverterFactory.create(gson))
            .client(okHttpClient);

    return retrofitBuilder.build();
  }

  private OkHttpClient buildHttpClient(final String apiKey, final int apiVersion) {
    final OkHttpClient.Builder builder = new OkHttpClient.Builder();
    builder.addInterceptor(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        final Request original = chain.request();
        final Request.Builder requestBuilder = original.newBuilder()
                .header(CloudConstants.MainHeaders.ACCEPT, "application/vnd.com.kontakt+json;version=" + apiVersion)
                .header(CloudConstants.MainHeaders.API_KEY, apiKey)
                .method(original.method(), original.body());

        // Add Kontakt Agent Header if printable
        final String kontaktAgentHeader = kontaktAgentHeader();
        if (kontaktAgentHeader != null) {
          requestBuilder.header(CloudConstants.MainHeaders.KONTAKT_AGENT, kontaktAgentHeader);
        }

        // Add Kontakt App Header if printable
        final String kontaktAppHeader = kontaktAppHeader();
        if (kontaktAppHeader != null) {
          requestBuilder.header(CloudConstants.MainHeaders.KONTAKT_APPLICATION, kontaktAppHeader);
        }

        final Request request = requestBuilder.build();
        return chain.proceed(request);
      }
    });
    return builder.build();
  }

  private String kontaktAgentHeader() {
    final String sdkVersion = BuildConfig.VERSION_NAME;
    final int systemVersion = Build.VERSION.SDK_INT;
    final String agentHeader = String.format(Locale.getDefault() , "sdk-%s-Android-%d", sdkVersion, systemVersion);

    if (!isAsciiPrintable(agentHeader)) {
      return null;
    }
    return agentHeader;
  }

  private String kontaktAppHeader() {
    if (context == null) {
      return null;
    }

    final String packageName = context.getPackageName();
    if (!isAsciiPrintable(packageName)) {
      return null;
    }
    return packageName;
  }

}

==> sdk-5.0.0-sources.jar: com/kontakt/sdk/android/cloud/KontaktCloudImpl.java <==
package com.kontakt.sdk.android.cloud;

import android.content.Context;
import android.os.Build;
import android.util.Log;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.kontakt.sdk.android.BuildConfig;
import com.kontakt.sdk.android.cloud.adapter.ActionTypeAdapter;
import com.kontakt.sdk.android.cloud.adapter.ConfigTypeAdapter;
import com.kontakt.sdk.android.cloud.adapter.DeviceTypeAdapter;
import com.kontakt.sdk.android.cloud.api.*;
import com.kontakt.sdk.android.cloud.api.service.*;

import com.kontakt.sdk.android.cloud.serialize.CoordinatesDeserializer;
import com.kontakt.sdk.android.cloud.serialize.CoordinatesSerializer;
import com.kontakt.sdk.android.cloud.util.ErrorUtils;
import com.kontakt.sdk.android.common.model.Action;
import com.kontakt.sdk.android.common.model.Config;
import com.kontakt.sdk.android.common.model.Coordinates;
import com.kontakt.sdk.android.common.model.Device;
import com.kontakt.sdk.android.common.util.SDKPreconditions;

import java.io.IOException;
import java.util.Locale;

import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import retrofit2.converter.scalars.ScalarsConverterFactory;

import static com.kontakt.sdk.android.cloud.util.StringUtils.isAsciiPrintable;

/**
 * Kontakt.io Proximity REST API client implementation.
 *
 * To obtain Kontakt Cloud instance please use {@link KontaktCloudFactory#create(String)} or
 * {@link KontaktCloudFactory#create()} method, for example:
 *
 * <pre>
 *   <code>
 *   KontaktCloud kontaktCloud = KontaktCloudFactory.create();
 *   KontaktCloud kontaktCloud = KontaktCloudFactory.create("my-api-key");
 *   </code>
 * </pre>
 */
@SuppressWarnings("WeakerAccess")
public class KontaktCloudImpl implements KontaktCloud {

  private final ServicesFactory servicesFactory;
  // used to get app package header value
  private Context context;

  KontaktCloudImpl(String apiKey, String apiUrl, int apiVersion) {
    SDKPreconditions.checkNotNullOrEmpty(apiKey, "Kontakt Cloud - empty api key.");
    SDKPreconditions.checkNotNullOrEmpty(apiUrl, "Kontakt Cloud - empty api url.");
    final Retrofit retrofit = buildRetrofit(apiUrl, buildHttpClient(apiKey, apiVersion));
    this.servicesFactory = ServicesFactory.newInstance(retrofit);
    ErrorUtils.initialize(retrofit);
  }

  private KontaktCloudImpl(Context context, String apiKey, String apiUrl, int apiVersion) {
    SDKPreconditions.checkNotNull(context, "Kontakt Cloud - null context.");
    SDKPreconditions.checkNotNullOrEmpty(apiKey, "Kontakt Cloud - empty api key.");
    SDKPreconditions.checkNotNullOrEmpty(apiUrl, "Kontakt Cloud - empty api url.");

    this.context = context;
    final Retrofit retrofit = buildRetrofit(apiUrl, buildHttpClient(apiKey, apiVersion));
    this.servicesFactory = ServicesFactory.newInstance(retrofit);
    ErrorUtils.initialize(retrofit);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ActionsApi actions() {
    return new ActionsApi(servicesFactory.createService(ActionsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DevicesApi devices() {
    return new DevicesApi(servicesFactory.createService(DevicesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public VenuesApi venues() {
    return new VenuesApi(servicesFactory.createService(VenuesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public PlacesApi places() {
    return new PlacesApi(servicesFactory.createService(PlacesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ReceiversApi receivers() {
    return new ReceiversApi(servicesFactory.createService(ReceiversService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ConfigsApi configs() {
    return new ConfigsApi(servicesFactory.createService(ConfigsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ManagersApi managers() {
    return new ManagersApi(servicesFactory.createService(ManagersService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public FirmwaresApi firmwares() {
    return new FirmwaresApi(servicesFactory.createService(FirmwaresService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ProximitiesApi proximities() {
    return new ProximitiesApi(servicesFactory.createService(ProximitiesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public NamespacesApi namespaces() {
    return new NamespacesApi(servicesFactory.createService(NamespacesService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public CommandsApi commands() {
    return new CommandsApi(servicesFactory.createService(CommandsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public EventsApi events() {
    return new EventsApi(servicesFactory.createService(EventsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public PresetsApi presets() {
    return new PresetsApi(servicesFactory.createService(PresetsService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public TriggersApi triggers() {
    return new TriggersApi(servicesFactory.createService(TriggersService.class));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ActivitiesApi activities() {
    return new ActivitiesApi(servicesFactory.createService(ActivitiesService.class));
  }

  private NotificationsApi notificationsApi() {
    return new NotificationsApi(servicesFactory.createService(NotificationsService.class));
  }

  private Retrofit buildRetrofit(final String apiBaseUrl, final OkHttpClient okHttpClient) {
    GsonBuilder gsonBuilder = new GsonBuilder();
    gsonBuilder.registerTypeAdapter(Device.class, new DeviceTypeAdapter());
    gsonBuilder.registerTypeAdapter(Config.class, new ConfigTypeAdapter());
    gsonBuilder.registerTypeAdapter(Action.class, new ActionTypeAdapter());
    gsonBuilder.registerTypeAdapter(Coordinates.class, new CoordinatesSerializer());
    gsonBuilder.registerTypeAdapter(Coordinates.class, new CoordinatesDeserializer());
    gsonBuilder.setPrettyPrinting();
    Gson gson = gsonBuilder.create();

    Retrofit.Builder retrofitBuilder = new Retrofit.Builder()
        .baseUrl(apiBaseUrl)
        .addConverterFactory(ScalarsConverterFactory.create())
        .addConverterFactory(GsonConverterFactory.create(gson))
        .client(okHttpClient);

    return retrofitBuilder.build();
  }

  private OkHttpClient buildHttpClient(final String apiKey, final int apiVersion) {
    final OkHttpClient.Builder builder = new OkHttpClient.Builder();
    builder.addInterceptor(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        final Request original = chain.request();
        final Request.Builder requestBuilder = original.newBuilder()
            .header(CloudConstants.MainHeaders.ACCEPT, "application/vnd.com.kontakt+json;version=" + apiVersion)
            .header(CloudConstants.MainHeaders.API_KEY, apiKey)
            .method(original.method(), original.body());

        // Add Kontakt Agent Header if printable
        final String kontaktAgentHeader = kontaktAgentHeader();
        if (kontaktAgentHeader != null) {
          requestBuilder.header(CloudConstants.MainHeaders.KONTAKT_AGENT, kontaktAgentHeader);
        }

        // Add Kontakt App Header if printable
        final String kontaktAppHeader = kontaktAppHeader();
        if (kontaktAppHeader != null) {
          requestBuilder.header(CloudConstants.MainHeaders.KONTAKT_APPLICATION, kontaktAppHeader);
        }

        final Request request = requestBuilder.build();
        return chain.proceed(request);
      }
    });
    builder.interceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Request request = chain.request();

        Response response = chain.proceed(request);
        String rawReponseJson = response.body().string();
        String rawRequestJson = request.toString();

        Log.d(BuildConfig.APPLICATION_ID, String.format("raw JSON request is: %s %s", rawRequestJson, request.url()));
        Log.d(BuildConfig.APPLICATION_ID, String.format("raw JSON response is: %s", rawReponseJson));

        return response.newBuilder()
                .body(ResponseBody.create(response.body().contentType(), rawReponseJson)).build();
      }
    });
    return builder.build();
  }

  private String kontaktAgentHeader() {
    final String sdkVersion = BuildConfig.VERSION_NAME;
    final int systemVersion = Build.VERSION.SDK_INT;
    final String agentHeader = String.format(Locale.getDefault(), "sdk-%s-Android-%d", sdkVersion, systemVersion);

    if (!isAsciiPrintable(agentHeader)) {
      return null;
    }
    return agentHeader;
  }

  private String kontaktAppHeader() {
    if (context == null) {
      return null;
    }

    final String packageName = context.getPackageName();
    if (!isAsciiPrintable(packageName)) {
      return null;
    }
    return packageName;
  }

}
